clc
clear all
clearvars
% close all

addpath(genpath('../MATLAB'))
addpath(genpath('../results'))

%% Read input files

% Get initial samples
initialSamples = dlmread('results/fableApophis/reintegrateInitialSamples.txt');
densitySamples = initialSamples(:,end);
initialSamples = initialSamples(:,1:end-1);

% Evolution (double and polynomial) of equinoctial state for refined and
% fable control
XrefinedDouble = dlmread('results/fableApophis/outputPropagatedSamplesRefinedDouble.txt');
XrefinedPoly   = dlmread('results/fableApophis/outputPropagatedSamplesRefinedPoly.txt');
XfableDouble   = dlmread('results/fableApophis/outputPropagatedSamplesFABLEDouble.txt');
XfablePoly     = dlmread('results/fableApophis/outputPropagatedSamplesFABLEPoly.txt');

% Extended time evolution according to refined double control
XrefinedDoubleExtended = dlmread('results/fableApophis/timeEvolutionRefined.txt');

% Bounds
Xbounds = dlmread('results/fableApophis/initialFinalMeanLBUB.txt');

%% Dimensions and extracted quantities

% Dimension
ns = 6;
nt = size(XrefinedDouble,2)/(ns);
nSamples = size(XrefinedDouble,1);

% Independent vector
s = XrefinedDouble(1,6:6:end);

% Mean and covariance of initial and target ellipsoids
initialMean = Xbounds(1,1:ns-1);
initialLB   = Xbounds(1,ns:2*(ns-1));
initialUB   = Xbounds(1,2*(ns-1)+1:3*(ns-1));
% finalMean   = Xbounds(2,1:ns-1);
finalLB     = Xbounds(2,ns:2*(ns-1));
finalUB     = Xbounds(2,2*(ns-1)+1:3*(ns-1));
finalMean   = (finalUB+finalLB)/2;
finalSigma  = (finalUB-finalLB)/2;


% Scale btw [0,1]
if (length(densitySamples)>1)
    dSSc = (densitySamples-min(densitySamples))./(max(densitySamples)-min(densitySamples));
else
    dSSc = densitySamples;
end
% Use colormap length
figure;
cm = colormap('jet');
close;
dSSc = floor(dSSc.*(size(cm,1)-1))+1;

% Arrange for increasing density
[dSScSorted,sortIndex] = sort(dSSc);


% %% Plot initial set of samples 
% 
% % Plot of final set projection on variables i and j
% % Generate arrival ellipse
% initialSamplesSorted = initialSamples(sortIndex,:);
% 
% figure (1)
% hold on
% pc = 1;
% for i = 1:ns-2
%     for j = i+1:ns-1
%         
%         % Which sublopt
%         subplot(2,5,pc);
%         pc = pc+1;                      
%         hold on
%                 
%         
%         
%         % Plot samples
%         for k=1:nSamples
%             plot(initialSamplesSorted(k,i),initialSamplesSorted(k,j), ...
%                 'Marker','o','MarkerFaceColor',cm(dSScSorted(k),:),'MarkerEdgeColor',cm(dSScSorted(k),:),'MarkerSize',6)
%         end
% %         plot(refP(:,i),refP(:,j),'k*')
% %         plot(fabD(:,i),fabD(:,j),'y*')
% %         plot(fabP(:,i),fabP(:,j),'y*')
%         
% %         axis equal
%         hold off
%         if (i==1)
%             xlabel('a [au]');
%         elseif (i==2)
%             xlabel('P1 [rad]')
%         elseif (i==3)
%             xlabel('P2 [rad]')
%         elseif (i==4)
%             xlabel('Q1 [rad]')
%         elseif (i==5)
%             xlabel('Q2 [rad]')
%         end
%         if (j==1)
%             ylabel('a [au]');
%         elseif (j==2)
%             ylabel('P1 [rad]')
%         elseif (j==3)
%             ylabel('P2 [rad]')
%         elseif (j==4)
%             ylabel('Q1 [rad]')
%         elseif (j==5)
%             ylabel('Q2 [rad]')
%         end
%         
%         
%     end
% end
% 



% %% Plot final set of samples with ellipsoid projections
% 
% % Plot of final set projection on variables i and j
% % Generate arrival ellipse
% 
% State to plot
tI = nt;
refD = XrefinedDouble(:,(tI-1)*ns+1:tI*ns);
refP = XrefinedPoly(:,(tI-1)*ns+1:tI*ns);
fabD = XfableDouble(:,(tI-1)*ns+1:tI*ns);
fabP = XfablePoly(:,(tI-1)*ns+1:tI*ns);

% Arrange for increasing density
refD = refD(sortIndex,:);
refP = refP(sortIndex,:);
fabD = fabD(sortIndex,:);
fabP = fabP(sortIndex,:);
% 
% % figure (2)
% figure 
% hold on
% pc = 1;
% for i = 1:ns-2
%     for j = i+1:ns-1
%         
%         % Which sublopt
%         subplot(2,5,pc);
%         pc = pc+1;                      
%         hold on
%        
%         % Ellipsoid
%         t = linspace(0,2*pi,1000);
%         xEl = finalMean(i)+finalSigma(i).*cos(t);
%         yEl = finalMean(j)+finalSigma(j).*sin(t);        
%         plot(xEl,yEl,'g--','LineWidth',3)
%         
%         % Plot samples refined double
%         plot(refD(:,i),refD(:,j),'LineStyle','none',...
%             'Marker','*','MarkerFaceColor','b','MarkerEdgeColor','b','MarkerSize',6);
%         
%         % Plot samples fable double
%         plot(fabD(:,i),fabD(:,j),'LineStyle','none',...
%             'Marker','*','MarkerFaceColor','y','MarkerEdgeColor','y','MarkerSize',6);
%             
% %         for k=1:nSamples
% %             plot(refD(k,i),refD(k,j), ...
% %                 'Marker','o','MarkerFaceColor',cm(dSScSorted(k),:),'MarkerEdgeColor',cm(dSScSorted(k),:),'MarkerSize',6)
% %             plot(refD(k,i),refD(k,j),...
% %                 'Marker','o','MarkerFaceColor',cm(dSScSorted(k),:),'MarkerEdgeColor',cm(dSScSorted(k),:),'MarkerSize',6)
% %         end
% %         plot(refP(:,i),refP(:,j),'k*')
% %         plot(fabD(:,i),fabD(:,j),'y*')
% %         plot(fabP(:,i),fabP(:,j),'y*')
%         
% %         axis equal
%         hold off
%         if (i==1)
%             xlabel('a [au]');
%         elseif (i==2)
%             xlabel('P1 [rad]')
%         elseif (i==3)
%             xlabel('P2 [rad]')
%         elseif (i==4)
%             xlabel('Q1 [rad]')
%         elseif (i==5)
%             xlabel('Q2 [rad]')
%         end
%         if (j==1)
%             ylabel('a [au]');
%         elseif (j==2)
%             ylabel('P1 [rad]')
%         elseif (j==3)
%             ylabel('P2 [rad]')
%         elseif (j==4)
%             ylabel('Q1 [rad]')
%         elseif (j==5)
%             ylabel('Q2 [rad]')
%         end
%         
%         
%         
%     end
% end
% legend('Target ellipsoid','Refined traj.','Guess traj.')
% 
% 
% 
% %% Plot final set of samples with ellipsoid projections with polynomial vs numerical propagation
% 
% % Plot of final set projection on variables i and j
% % Generate arrival ellipse
% 
% figure
% hold on
% pc = 1;
% for i = 1:ns-2
%     for j = i+1:ns-1
%         
%         % Which sublopt
%         subplot(2,5,pc);
%         pc = pc+1;                      
%         hold on
%        
%         % Ellipsoid
%         t = linspace(0,2*pi,1000);
%         xEl = finalMean(i)+finalSigma(i).*cos(t);
%         yEl = finalMean(j)+finalSigma(j).*sin(t);        
%         plot(xEl,yEl,'g--','LineWidth',3)
%         
%         % Plot samples refined double
%         plot(refD(:,i),refD(:,j),'LineStyle','none',...
%             'Marker','*','MarkerFaceColor','b','MarkerEdgeColor','b','MarkerSize',6);
%         
%         % Plot samples fable double
%         plot(refP(:,i),refP(:,j),'LineStyle','none',...
%             'Marker','*','MarkerFaceColor','r','MarkerEdgeColor','r','MarkerSize',6);
%             
% %         for k=1:nSamples
% %             plot(refD(k,i),refD(k,j), ...
% %                 'Marker','o','MarkerFaceColor',cm(dSScSorted(k),:),'MarkerEdgeColor',cm(dSScSorted(k),:),'MarkerSize',6)
% %             plot(refD(k,i),refD(k,j),...
% %                 'Marker','o','MarkerFaceColor',cm(dSScSorted(k),:),'MarkerEdgeColor',cm(dSScSorted(k),:),'MarkerSize',6)
% %         end
% %         plot(refP(:,i),refP(:,j),'k*')
% %         plot(fabD(:,i),fabD(:,j),'y*')
% %         plot(fabP(:,i),fabP(:,j),'y*')
%         
% %         axis equal
%         hold off
%         if (i==1)
%             xlabel('a [au]');
%         elseif (i==2)
%             xlabel('P1 [rad]')
%         elseif (i==3)
%             xlabel('P2 [rad]')
%         elseif (i==4)
%             xlabel('Q1 [rad]')
%         elseif (i==5)
%             xlabel('Q2 [rad]')
%         end
%         if (j==1)
%             ylabel('a [au]');
%         elseif (j==2)
%             ylabel('P1 [rad]')
%         elseif (j==3)
%             ylabel('P2 [rad]')
%         elseif (j==4)
%             ylabel('Q1 [rad]')
%         elseif (j==5)
%             ylabel('Q2 [rad]')
%         end
%                
%     end
% end
% legend('Target ellipsoid','Refined numerical','Refined polynomial')





%% Plot final set of samples with density

% Plot of final set projection on variables i and j
% Generate arrival ellipse

figure 
hold on
pc = 1;
for i = 1:ns-2
    for j = i+1:ns-1
        
        % Which sublopt
        subplot(2,5,pc);
        pc = pc+1;                      
        hold on
        
        % Ellipsoid
        t = linspace(0,2*pi,1000);
        xEl = finalMean(i)+finalSigma(i).*cos(t);
        yEl = finalMean(j)+finalSigma(j).*sin(t);        
        plot(xEl,yEl,'g--','LineWidth',3)        
        
        % Plot samples
%         plot(refD(:,i),refD(:,j), ...
%                 'Marker','o','MarkerEdgeColor','b','LineStyle','none','MarkerSize',6)
        for k=1:nSamples
            plot(refD(k,i),refD(k,j), ...
                'Marker','o','MarkerFaceColor',cm(dSScSorted(k),:),'MarkerEdgeColor',cm(dSScSorted(k),:),'MarkerSize',6)
        end
%         plot(refP(:,i),refP(:,j),'k*')
%         plot(fabD(:,i),fabD(:,j),'y*')
%         plot(fabP(:,i),fabP(:,j),'y*')
        
%         axis equal
        hold off
        if (i==1)
            xlabel('a [au]');
        elseif (i==2)
            xlabel('P1 [rad]')
        elseif (i==3)
            xlabel('P2 [rad]')
        elseif (i==4)
            xlabel('Q1 [rad]')
        elseif (i==5)
            xlabel('Q2 [rad]')
        end
        if (j==1)
            ylabel('a [au]');
        elseif (j==2)
            ylabel('P1 [rad]')
        elseif (j==3)
            ylabel('P2 [rad]')
        elseif (j==4)
            ylabel('Q1 [rad]')
        elseif (j==5)
            ylabel('Q2 [rad]')
        end
        
    end
end

