clc
clear all
% close all

addpath(genpath('/home/cristian/Desktop/dev/SMART/smart-astro/MATLAB/'))
addpath(genpath('../results'))


mu = 1.32712440018e20;
au = 149597870.7e3;

Lscale = au;
Tscale = 1;
% mu = mu / Lscale^3 * Tscale^2;

%% Read input files

% Get initial samples
initialSamples = dlmread('results/fableApophis/reintegrateInitialSamples.txt');
densitySamples = initialSamples(:,end);
initialSamples = initialSamples(:,1:end-1);

% Evolution (double and polynomial) of equinoctial state for refined and
% fable control
EqRefinedDouble = dlmread('results/fableApophis/outputPropagatedSamplesRefinedPoly.txt');
CarRefinedDouble = dlmread('results/fableApophis/outputPropagatedSamplesRefinedPolyCartesian.txt');
% XrefinedPoly   = dlmread('results/fableApophis/outputPropagatedSamplesRefinedPoly.txt');
% XfableDouble   = dlmread('results/fableApophis/outputPropagatedSamplesFABLEDouble.txt');
% XfablePoly     = dlmread('results/fableApophis/outputPropagatedSamplesFABLEPoly.txt');

% Extended time evolution according to refined double control
% XrefinedDoubleExtended = dlmread('results/fableApophis/timeEvolutionRefined.txt');

% Bounds
Xbounds = dlmread('results/fableApophis/initialFinalMeanLBUB.txt');

%% Dimensions and extracted quantities

% Dimension
ns = 6;
nt = size(EqRefinedDouble,2)/(ns);
nSamples = size(EqRefinedDouble,1);

% Independent vector
s = EqRefinedDouble(1,6:6:end);

% Mean and covariance of initial and target ellipsoids
initialMean = Xbounds(1,1:ns-1);
initialLB   = Xbounds(1,ns:2*(ns-1));
initialUB   = Xbounds(1,2*(ns-1)+1:3*(ns-1));
% finalMean   = Xbounds(2,1:ns-1);
finalLB     = Xbounds(2,ns:2*(ns-1));
finalUB     = Xbounds(2,2*(ns-1)+1:3*(ns-1));
finalMean   = (finalUB+finalLB)/2;
finalSigma  = (finalUB-finalLB)/2;

%% Final state of Apophis

eqApophis = [finalMean(:) ; s(end)];
eqApophis(1) = eqApophis(1)*Lscale; % Convert to meters
carApophis = eq2cart_m(eqApophis,mu);

%% Set of final controlled samples
tI = nt;
carRefD = CarRefinedDouble(:,(tI-1)*ns+1:tI*ns);

% eqRefD = EqRefinedDouble(:,(tI-1)*ns+1:tI*ns);
% eqRefD(:,1) = eqRefD(:,1)*Lscale; % Convert to meters
% 
% % convert to cartesian
% carRefD = eqRefD;
% for i = 1:nSamples
%     carRefD(i,:) = eq2cart_m(eqRefD(i,:),mu)';
% end

%% Compute minimum, maximum and average distance
DR     = carRefD(:,1:3)-carApophis(1:3)';
DR     = sort(sqrt( DR(:,1).^2+DR(:,2).^2+DR(:,3).^2 )/1.0e3); % km
minDR  = DR(1);
maxDR  = DR(end);
meanDR = mean(DR);

DV     = carRefD(:,4:6)-carApophis(4:6)';
DV     = sort(sqrt( DV(:,1).^2+DV(:,2).^2+DV(:,3).^2 )/1.0e3); % km/s
minDV  = DV(1);
maxDV  = DV(end);
meanDV = mean(DV);


%% Plot DR and DV for samples

p = 0.95;

figure

subplot(2,1,1)
hold on
title('\Delta R')
plot(1:1:nSamples,DR,'*k','Linestyle','none')
plot(floor(p*nSamples)*[1 1],[minDR maxDR],'r','Linewidth',2)
xlabel('Index')
ylabel('\Delta R [km]')

subplot(2,1,2)
hold on
title('\Delta V')
plot(1:1:nSamples,DV,'*k','Linestyle','none')
plot(floor(p*nSamples)*[1 1],[minDV maxDV],'r','Linewidth',2)
xlabel('Index')
ylabel('\Delta V [km/s]')


%% Empirical cdf
figure
ecdf(DR)

%% Empirical cdf custom

