clc
clear all
close all

addpath(genpath('/home/cristian/Desktop/dev/SMART/smart-astro/MATLAB/'))
addpath(genpath('../results'))

mu = 1.32712440018e20;
au = 149597870.7e3;

Lscale = au;
% Tscale = 1;
% mu = mu / Lscale^3 * Tscale^2;

%% Read input files

vInitialSamples = dlmread('initialVSamplesAndPdf.csv');
densitySamplesSampled = dlmread('reintegrateInitialSamples.txt');
EqRefinedDouble = dlmread('outputPropagatedSamplesRefinedPoly.txt');
carRefD = dlmread('finalCartesianAndPDF.csv');
carRefDSampled = dlmread('outputPropagatedSamplesRefinedPolyCartesian.txt');
Xbounds = dlmread('initialFinalMeanLBUB.txt');


%% Dimensions and extracted quantities

% Dimension
ns = 6;
nt = size(EqRefinedDouble,2)/(ns);

nSamples = size(carRefD,1);
nSamplesSampled = size(carRefDSampled,1);

% Mean and covariance of initial and target ellipsoids
% initialMean = Xbounds(1,1:ns-1);
% initialLB   = Xbounds(1,ns:2*(ns-1));
% initialUB   = Xbounds(1,2*(ns-1)+1:3*(ns-1));
% finalMean   = Xbounds(2,1:ns-1);
finalLB     = Xbounds(2,ns:2*(ns-1));
finalUB     = Xbounds(2,2*(ns-1)+1:3*(ns-1));
finalMean   = (finalUB+finalLB)/2;
% finalSigma  = (finalUB-finalLB)/2;


%% Samples

% Get initial samples' density
densitySamplesSampled = densitySamplesSampled(:,end);

% Samples from original distribution 
carRefDSampled = carRefDSampled(:,(nt-1)*ns+1:nt*ns)./1.0e3; % km, km/s

% Independent vector
s = EqRefinedDouble(1,6:6:end);
clear EqRefinedDouble;

% Sobol samples in velocity and density
densitySamples = vInitialSamples(:,end);
vInitialSamples = vInitialSamples(:,1:end-1);

% Scale density by importance sampling
importanceSamplingFactor = (100.0) * (2*1*pi/180) * (2*0.5*pi/180);
densitySamples = densitySamples * importanceSamplingFactor;

% Sobol samples propagated
carRefD = carRefD(:,1:end-1)./1.0e3; % km, km/s


%% Final state of Apophis

eqApophis = [finalMean(:) ; s(end)];
eqApophis(1) = eqApophis(1)*Lscale; % Convert to meters
carApophis = eq2cart_m(eqApophis,mu);
carApophis = carApophis(:)'/1.0e3; % km, km/s


%% Compute minimum, maximum and average distance
DR = carRefD(:,1:3)-carApophis(1:3);
DR = sqrt( DR(:,1).^2+DR(:,2).^2+DR(:,3).^2 );

% DV     = carRefD(:,4:6)-carApophis(4:6)';
% DV     = sqrt( DV(:,1).^2+DV(:,2).^2+DV(:,3).^2 );

DRSampled = carRefDSampled(:,1:3)-carApophis(1:3);
DRSampled = sqrt( DRSampled(:,1).^2+DRSampled(:,2).^2+DRSampled(:,3).^2 );

% DVSampled     = carRefDSampled(:,4:6)-carApophis(4:6)';
% DVSampled     = sqrt( DVSampled(:,1).^2+DVSampled(:,2).^2+DVSampled(:,3).^2 ); 


%% Plot DR and DV for samples
% 
p = 0.95;

% figure
% 
% subplot(2,1,1)
% hold on
% title('\Delta R')
% plot(1:1:nSamples,sort(DR),'*k',1:1:nSamples,sort(DRSampled),'*g','Linestyle','none')
% plot(floor(p*nSamples)*[1 1],[min(DR) max(DR)],'r',floor(p*nSamples)*[1 1],[min(DRSampled) max(DRSampled)],'g','Linewidth',2)
% xlabel('Index')
% ylabel('\Delta R [km]')


% 
% subplot(2,1,2)
% hold on
% title('\Delta V')
% plot(1:1:nSamples,DV,'*k','Linestyle','none')
% plot(floor(p*nSamples)*[1 1],[minDV maxDV],'r','Linewidth',2)
% xlabel('Index')
% ylabel('\Delta V [km/s]')

% 
% %% Empirical cdf
% figure
% grid on
% hold on
% % ecdf(DR)
% ecdf(DRSampled)
% 
% 
% %% Custom cds
% 
% % figure
% 
% % Sobol sampled
% [xi,cumProbi] = ecdfCustom(DR,densitySamples);
% plot(xi,cumProbi,'--g')
% 
% % Samples from original distribution
% [xiSampled,cumProbiSampled] = ecdfCustom(DRSampled,ones(length(DRSampled),1));
% plot(xiSampled,cumProbiSampled,'--k')
% 
% % legend('ecdf','ecdfSampled','cumProb','cumProbSampled')
% legend('ecdfSampled','cumProb','cumProbSampled')
% 
% 
% %% Check means
% % weightedMeanDR = sum(densitySamples.* DR)/sum(densitySamples)
% % weightedMeanDRSampled = sum(densitySamplesSampled.* DRSampled)/sum(densitySamplesSampled)
% 
% 
% 
% % mean(DRSampled)
% % sum(DRSampled.*densitySamplesSampled)/sum(densitySamplesSampled)
% % 
% ExpectedValue        = sum(DR.*densitySamples)/nSamples
% ExpectedValueSampled = mean(DRSampled)
% % Integral = Integral * (100.0/1.0e3) * (2.0*pi/180) * (2.0*pi/180)
% 
% Thres = 3.0e5;
% % DRSmallerThres = DR( DR<=Thres );
% densitySmallerThres = densitySamples( DR<=Thres );
% % DRSmallerThresSam = DRSampled( DRSampled<=Thres );
% 
% ProbSmallerThres = sum(densitySmallerThres)/nSamples
% ProbSmallerThresSampled = sum(DRSampled<=Thres)/nSamplesSampled


%% Reconstruct sample densities

% Number of pdf to create
nPdf = 16;
dens = ones(nSamples,nPdf);


% Family of optimised distribution
nPdf1 = 15;

% distributions "mean"
a_v_inf     = 0.0;
a_alpha_inf = 0.0;
a_beta_inf  = 0.0;

% distribution scaling for sigma
pHalton = haltonset(3);
Scaling = net(pHalton,nPdf1-1);

% Scale scaling
Scaling = Scaling * 2.0 + 3.0;
Scaling = [[4.0 4.0 4.0]; Scaling];

% Sigma
sigma_v_inf     = 100.0./Scaling(:,1);
sigma_alpha_inf = 1.0*pi/180./Scaling(:,2);
sigma_beta_inf  = 0.5*pi/180./Scaling(:,3);

% Compute density
for i = 1:nPdf1
    
    dens(:,i) = ran_gaussian_tail_pdf(-vInitialSamples(:,1),a_v_inf,sigma_v_inf(i)) .* ...
                    normpdf(vInitialSamples(:,2),a_alpha_inf,sigma_alpha_inf(i)) .* ...
                    normpdf(vInitialSamples(:,3),a_beta_inf,sigma_beta_inf(i)) ;

end



% Uniform distribution
nPdf2 = 1;

% Compute density
for i = nPdf1+1:nPdf1+nPdf2    
    dens(:,i) = 1/100.0 * ...
                1/(2*pi/180) * ...
                1/(1*pi/180) ;
end

% 


% Scale by importance sampling
dens = dens * importanceSamplingFactor;


%% Empirical cdf for each density
figure
grid on
hold on
[cumProbi,xi] = ecdf(DRSampled);
plot(xi,cumProbi,'k','Linewidth',3)

legendStr = {'Reference Ecdf'};

% Sobol sampled same family as optimised
for i = 1:nPdf
    [xi,cumProbi] = ecdfCustom(DR,dens(:,i));
    plot(xi,cumProbi,'--')    
end

% Legend
for i = 1:nPdf1
    legendStr{i+1} = ['\sigma_v = ' num2str(sigma_v_inf(i),'%5.3f') '; \sigma_{\alpha} = ' num2str(sigma_alpha_inf(i),'%5.3f') '; \sigma_{\beta} = ' num2str(sigma_beta_inf(i),'%5.3f') ];
end 
for i = nPdf1+1:nPdf1+nPdf2
    legendStr{i+1} = ['Uniform'];
end

% legend('ecdf','ecdfSampled','cumProb','cumProbSampled')
legend(legendStr)
