clc
clear all
close all

addpath(genpath('../results'));

%% Quantities of planar orbital problem
ns = 3;
nc = 2;
deg = 5;

%% Read initial mean and sigma
R = dlmread('initialFinalMeanLBUB.txt');

initialMean = R(1,1:3)';
initialLB = R(1,4:6)';
initialUB = R(1,7:9)';
initialSigma = (initialUB-initialLB)./2;

finalMean = R(2,1:3)';
finalLB = R(2,4:6)';
finalUB = R(2,7:9)';
finalSigma = (finalUB-finalLB)./2;

clear R;


%% Read results from fable and refined trajectory
XoptDouble = dlmread('outputPropagatedSamplesOptimalControlDouble.txt');
XoptPoly = dlmread('outputPropagatedSamplesOptimalControlPoly.txt');
XfabDouble = dlmread('outputPropagatedSamplesFABLEDouble.txt');
XfabPoly = dlmread('outputPropagatedSamplesFABLEPoly.txt');

% Number of L discretisation points
nL = size(XoptDouble,2)/(ns+1);
nSamples = size(XoptDouble,1);
L = zeros(nL,1);
for i=1:nL
   L(i) = XoptDouble(1,4*i); 
end

%% Compute control DV
Uopt = dlmread(['outputRefinedControlDeg' num2str(deg) '.txt']);
Ufab = dlmread(['outputFableControlDeg' num2str(deg) '.txt']);

DVopt = 0;
DVfab = 0;
for i = 1:nL-1
    DVopt = DVopt + Uopt(i,1)*(L(i+1)-L(i));
    DVfab = DVfab + Ufab(i,1)*(L(i+1)-L(i));
end
DVopt
DVfab
% keyboard
%% Plot Final Regions


% Final state
XFoptDouble = XoptDouble(:,end-ns:end);
XFoptPoly = XoptPoly(:,end-ns:end);
XFfabDouble = XfabDouble(:,end-ns:end);
XFfabPoly = XfabPoly(:,end-ns:end);

% Plot of final set projection on variables i and j
% Generate arrival ellipse
figure
hold on
pc = 1;

for i = 1:ns-1
    for j = i+1:ns
        
        subplot(1,3,pc);
        pc = pc+1;
        
        t = linspace(0,2*pi,1000);
        xEl = finalMean(i)+finalSigma(i).*cos(t);
        yEl = finalMean(j)+finalSigma(j).*sin(t);
        
        % Plot ellipsoid
        hold on
        plot(xEl,yEl,'b','LineWidth',3)
        plot(finalMean(i),finalMean(j)-finalSigma(j),'r*','Linewidth',5)
        plot(finalMean(i),finalMean(j)+finalSigma(j),'r*','Linewidth',5)
        plot(finalMean(i)-finalSigma(i),finalMean(j),'r*','Linewidth',5)
        plot(finalMean(i)+finalSigma(i),finalMean(j),'r*','Linewidth',5)
        
        % Plot samples
        plot(XFoptDouble(:,i),XFoptDouble(:,j),'y*')
        plot(XFoptPoly(:,i),XFoptPoly(:,j),'g*')
        
        plot(XFfabDouble(:,i),XFfabDouble(:,j),'k*')
        plot(XFfabPoly(:,i),XFfabPoly(:,j),'b*')
        
        
        
%         axis equal
%         hold off
        if (i==1)
            xlabel('a [au]');
        elseif (i==2)
            xlabel('P1 [rad]')
        elseif (i==3)
            xlabel('P2 [rad]')
        end
        if (j==1)
            ylabel('a [au]');
        elseif (j==2)
            ylabel('P1 [rad]')
        elseif (j==3)
            ylabel('P2 [rad]')
        end
        
        
    end
end



%% Plot state evolution as function of L
% 
figure
title('State evolution')
pc = ns;

for i = 1:pc
    
    indexState = [i:ns+1:(ns+1)*nL]';
    
    subplot(1,3,i);
    hold on
    for j = 1:nSamples
        % Plot samples
        plot(L,XoptDouble(j,indexState),'y*')
        plot(L,XoptPoly(j,indexState),'g*')
        
        plot(L,XfabDouble(j,indexState),'k*')
        plot(L,XfabPoly(j,indexState),'b*')
    end
    xlabel('L [rad]')
    if (i==1)
        ylabel('a [au]');
    elseif (i==2)
        ylabel('P1 [rad]')
    elseif (i==3)
        ylabel('P2 [rad]')
    end
    hold off
    
end


%% Plot controls
figure
title('Controls')

% Magnitude
subplot(1,2,1)
hold on
xlabel('L [rad]')
ylabel('u [au/tsid2]')
for i=1:nL-1
    plot([L(i) L(i+1)],[Uopt(i,1) Uopt(i,1)],'g','LineWidth',3)
    plot([L(i) L(i+1)],[Ufab(i,1) Ufab(i,1)],'b--','LineWidth',3)
end
hold off

% Angle
subplot(1,2,2)
hold on
xlabel('L [rad]')
ylabel('\alpha [rad]')
for i=1:nL-1
    plot([L(i) L(i+1)],[Uopt(i,2) Uopt(i,2)],'g','LineWidth',3)
    plot([L(i) L(i+1)],[Ufab(i,2) Ufab(i,2)],'b--','LineWidth',3)
end
hold off



% %% Final complying points
% invSigma = zeros(ns,ns);
% for i = 1:ns
%     invSigma(i,i) = 1/finalSigma(i)^2;
% end
% nNonEllipsoidFab = 0;
% nNonEllipsoidOpt = 0;
% for i = 1:nSamples
%     xfab = (XFfab(i,1:ns)-finalMean)';
%     xopt = (XFopt(i,1:ns)-finalMean)';
%     yfab = xfab'*invSigma*xfab;
%     yopt = xopt'*invSigma*xopt;
%     if (yfab>1)
%         nNonEllipsoidFab = nNonEllipsoidFab+1;
%     end
%     if (yopt>1)
%         nNonEllipsoidOpt = nNonEllipsoidOpt+1;
%     end
% end
% disp(['percNonEllipsidFav = ' num2str(nNonEllipsoidFab/nSamples)])
% disp(['percNonEllipsidOpt = ' num2str(nNonEllipsoidOpt/nSamples)])