clc
clear all
close all

addpath(genpath('../../../SMART/smart-astro/MATLAB/'))

% Constants
au = 149597870700;
km = 1e3;
mu = 1.32712440018e20;

% Scale
L_scale = 1;
T_scale = 1;
mu = mu / (L_scale^3) * (T_scale^2);

% Quantities
v_infRef = 3.34e3/L_scale*T_scale; % [m/s]
alpha_infRef = 0.613804119180597; % [rad]

%Earth
xEarthEq = [149598931569.775848/L_scale 0.016247 -0.003864 0.000000 0.000000 0.499269]';
xEarthKep = eq2kep(xEarthEq);
xEarthCar = kep2cart(xEarthKep,mu);

keyboard

%S/c reference
xScEqRef = [154981417672.346069/L_scale -0.070260 0.072689 0.000000 0.000000 0.499269]';
xScKepRef = eq2kep(xScEqRef);
xScCarRef = kep2cart(xScKepRef,mu);




% Uncertainty in initial DV wrt ref bounds
d2r = pi/180;
v_infLBUB = [-100.0 0.0]/L_scale*T_scale; %[m/s
alpha_infLBUB = [-1.0*d2r +1.0*d2r]; %[m/s


%S/c recomputed
xScCar = xEarthCar;
xScCar(4) = xScCar(4)+v_infRef*cos(alpha_infRef);
xScCar(5) = xScCar(5)+v_infRef*sin(alpha_infRef);
xScKep = cart2kep(xScCar,mu);
xScEq = kep2eq(xScKep);

xScCarRef
xScCar

xScEqRef
xScEq


% Sample generation
nSamples = 10000;
uncSamples = zeros(10000,2);

% Velocity uncertainty
pdV = makedist('Weibull');
% Truncate
pdV = truncate(pdV,0,5);

% % Alpha uncertainty
% pdV = makedist('Normal');
% % Truncate
% pdV = truncate(pdV,-5,+5);

%Generate random numbers from the distribution.
rng('default')  % For reproducibility
uncSamples(:,1) = -1*random(pdV,nSamples,1)/5*abs(v_infLBUB(2)-v_infLBUB(1));
uncSamples(1,1) = 0;

% Alpha uncertainty
pdA = makedist('Normal');
% Truncate
pdA = truncate(pdA,-3,+3);

%Generate random numbers from the distribution.
% rng('default')  % For reproducibility
uncSamples(:,2) = random(pdA,nSamples,1)/3*abs(alpha_infLBUB(2)-alpha_infLBUB(1))/2;
uncSamples(1,2) = 0;

% figure
% histfit(uncSamples(:,1),100)
% 
% figure
% histfit(uncSamples(:,2),100)
% 
% keyboard

% Compute equinoctial for every sample
uncertainEq = zeros(nSamples,6);
for i = 1:nSamples

   % Perturbed inf
   v_inf = v_infRef + uncSamples(i,1);
   alpha_inf = alpha_infRef + uncSamples(i,2);

   % Conversion to equinoctial
   cart = xEarthCar;
   cart(4) = cart(4)+v_inf*cos(alpha_inf);
   cart(5) = cart(5)+v_inf*sin(alpha_inf);
   kep = cart2kep(cart,mu);
   uncertainEq(i,:) = kep2eq(kep);
end

figure
subplot(1,3,1)
plot([1:1:nSamples]',uncSamples(:,1),'r*')
ylabel('v_{inf}')
subplot(1,3,2)
plot([1:1:nSamples]',uncSamples(:,2),'r*')
ylabel('\alpha')
subplot(1,3,3)
plot(uncSamples(:,1),uncSamples(:,2),'r*')
xlabel('v_{inf}')
ylabel('\alpha')


% Plot uncertain region in equinoctial elements
figure
subplot(1,3,1)
hold on
plot(uncertainEq(:,1)/au,uncertainEq(:,2),'b*')
plot(xScEq(1)/au,xScEq(2),'rs','MarkerSize',5)
xlabel('a [au]')
ylabel('P1 [rad]')

subplot(1,3,2)
hold on
plot(uncertainEq(:,1)/au,uncertainEq(:,3),'b*')
plot(xScEq(1)/au,xScEq(3),'rs','MarkerSize',5)
xlabel('a [au]')
ylabel('P2 [rad]')

subplot(1,3,3)
hold on
plot(uncertainEq(:,2),uncertainEq(:,3),'b*')
plot(xScEq(2),xScEq(3),'rs','MarkerSize',5)
ylabel('P2 [rad]')
xlabel('P1 [rad]')