/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
/*
------ Copyright (C) 2018 University of Strathclyde and Authors ------
-------------------- e-mail: c.greco@strath.ac.uk --------------------
----------------------- Author: Cristian Greco -----------------------
*/

#ifndef IPANEMA_PROPAGATORSURROGATE_H
#define IPANEMA_PROPAGATORSURROGATE_H

#include <vector>
#include <functional>
#include <cmath>

#include "ipanemaException.h"

namespace ipanema
{
namespace propagatorSurrogates
{

    class propagatorSurrogate {

        /**
         * Parameters
         */

    public:
        
        struct propagatorSurrogateParams {
            // Nothing here
        };

        /**
         * Member list
         */

    protected:


        /**
         * Constructor and Destructor
         */

    public:

        // Default constructor
        propagatorSurrogate();

        // Default destructor
        virtual ~propagatorSurrogate();

        
        /**
         * List of virtual functions to be implemented in derived classes
         */

    public:

        // Evaluate the surrogate
        virtual std::vector<double> propagate (const std::vector<double>& x) const;

        // Build the surrogate
        virtual void build ();



        /**
         * Help routines to shorten code
         */

    protected:

        // Sanity checks on input parameters
        virtual void inputParamsSanityCheck() const;

    };

} // namespace propagatorSurrogates
} // namespace ipanema

#endif //IPANEMA_PROPAGATORSURROGATE_H
