/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
/*
------ Copyright (C) 2018 University of Strathclyde and Authors ------
-------------------- e-mail: c.greco@strath.ac.uk --------------------
----------------------- Author: Cristian Greco -----------------------
*/

#ifndef IPANEMA_SAMPLINGCONSTRAINTS_H
#define IPANEMA_SAMPLINGCONSTRAINTS_H

#include <vector>
#include <cmath>

#include "../ipanemaException.h"

namespace ipanema
{
namespace ipanemaConstraints
{


    class samplingConstraints {


        /**
         * Parameter struct
         */

    public:

        struct samplingConstraintsParams {

            // Samples for constraint evaluation
            std::vector<std::vector<double>>*        pSamples;

            // Use only saved subset of the samples [for derivative computation in NLP]
            bool                                     sampleSubsetOption;

            // Pointer to bool whether to use subset or full set [if sampleSubsetOption = true]
            bool*                                    pUseFullSampleSet;

        };



        /**
         * Class members
         */

    protected:

        // Parameters
        const samplingConstraintsParams*  m_pParams;

        // Vector of indeces for subset to use [if sampleSubsetOption = true]
        std::vector<unsigned int>         m_subsetIndex;



        /**
         * Constructor and Destructor
         */

    public:

        // Default constructor
        samplingConstraints(const samplingConstraintsParams* pParams);

        // Default destructor
        virtual ~samplingConstraints();



        /**
         * List of virtual functions to be implemented in derived classes
         */

    public:

        // Evaluate the constraint in pSamples
        virtual double evaluate ();


    }; // class samplingConstraints


} // namespace ipanemaConstraints
} // namespace ipanema

#endif //IPANEMA_SAMPLINGCONSTRAINTS_H
