/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
/*
------ Copyright (C) 2018 University of Strathclyde and Authors ------
-------------------- e-mail: c.greco@strath.ac.uk --------------------
----------------------- Author: Cristian Greco -----------------------
*/

#ifndef IPANEMA_INTEGRALOFCONTROL_H
#define IPANEMA_INTEGRALOFCONTROL_H

#include <cmath>
#include "deterministicObjectives.h"

namespace ipanema
{
namespace ipanemaObjectives
{

    class integralOfControl : public deterministicObjectives {

        /**
         * Control type enum
         */

    public:

        typedef enum
        {
            COMPONENT_CONTROL,          // Magnitude is sqrt(norm(control))
            MAGNITUDEANGLES_CONTROL,    // Magnitude is control[0]

            MAX_CONTROLTYPE
        } ControlType;

        typedef enum
        {
            MODULUS_OBJECTIVE,   // objective = int |control|
            SQUARE_OBJECTIVE,    // objective = int control^2

            MAX_OBJECTIVETYPE
        } ObjectiveIntegrandType;


        /**
         * Parameter struct
         */

    public:

        struct integralOfControlParams : public deterministicObjectivesParams {

            // Control parematers [constant control]
            const std::vector<std::vector<double>>*  pControlCoefficients;

            // Time discretisation (or any other independent variable)
            const std::vector<double>*               pTimeGrid;

            // Type of control
            ControlType                              controlType;

            // Type of integrand
            ObjectiveIntegrandType                   objectiveIntegrandType;

        };



        /**
         * Class members
         */

    protected:

        const integralOfControlParams* m_pParams;



        /**
         * Constructor
         */

    public:

        // Default constructor
        integralOfControl(const deterministicObjectivesParams* pParams);



        /**
         * List of virtual functions to be implemented in this derived class
         */

    public:

        // Evaluate the objective according to quantities in params
        double evaluate ();



        /**
         * List of private help functions to better organise the code
         */

    private:

        // Integrand of integral at discretization interval discretisationIndex
        double evaluateIntegrand (unsigned int discretisationIndex) const;

    };


} // namespace ipanemaObjectives
} // namespace ipanema

#endif //IPANEMA_INTEGRALOFCONTROL_H
