/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
/*
------ Copyright (C) 2018 University of Strathclyde and Authors ------
-------------------- e-mail: c.greco@strath.ac.uk --------------------
----------------------- Author: Cristian Greco -----------------------
*/

#include "utils.h"

using namespace ipanema;

/**
 * Function to read from file
 */
int ipanema::dlmread ( const std::string &file_name,
                       std::vector<std::vector<double>>& output,
                       const char& delim ) {

    std::ifstream input ;
    input.open(file_name);

    if (!input.is_open())
        ipanema_throw("File " + file_name + "  not correctly open");

    std::string line ;
    output.clear();
    unsigned int line_counter = 0;
    while ( std::getline(input,line) ) {

        std::vector<double> row;
        std::istringstream iss(line);
        std::string token;

        while (std::getline(iss, token,delim)) {
            row.push_back(std::stod(token));
        }
        output.push_back(row);

        line_counter++;
    }
    input.close();

    return 0;
}


/**
 * Function to write to file
 */
int ipanema::dlmwrite ( const std::string &file_name,
                        const std::vector<std::vector<double>> &output,
                        const unsigned int &precision ,
                        const std::string &delimiter ) {

    std::ofstream output_stream;
    output_stream.open(file_name);

    if (!output_stream.is_open())
        ipanema_throw("File " + file_name + " not correctly open");

    unsigned int li_output = output.size();
    for (unsigned int i = 0; i < li_output; i++) {
        unsigned int lj_output = output[i].size();
        for (unsigned int j = 0; j < lj_output; j++) {

            output_stream << std::setprecision(precision) << output[i][j] ;

            if ( j!=lj_output-1 )
                output_stream << delimiter;
        }
        output_stream << "\n";
    }
    output_stream.close();

    return 0;
}

